/***************************************************************************
 *
 * Copyright (C) 2001 International Business Machines
 * All rights reserved.
 *
 * This file is part of the GPFS mmfslinux kernel module.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer. 
 *  2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 *  3. The name of International Business Machines may not be used to endorse 
 *     or promote products derived from this software without specific prior 
 *     written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Alternatively, provided that this notice is retained in full, this
 * software may be distributed under the terms of the GNU General
 * Public License ("GPL") as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version,
 * in which case the provisions of the GPL apply INSTEAD OF those given above.
 *
 ****************************************************************************/
/* @(#)69       1.86.1.2  src/avs/fs/mmfs/ts/DirIds.h, mmfs, avs_rhin, rhins008a 8/17/11 13:05:09 */
#ifndef _h_DirIds
#define _h_DirIds

/*
 * Define unique numeric IDs for each source directory in Shark that contains
 * code.  These directory IDs are used for several purposes:
 *   1. Generating unique labels ("hookids") for kernel trace points
 *   2. Identifying message set numbers for the message catalog code
 * The symbols defined in this file are formed by appending the relative
 * directory name to the string "DIR_ID_", then translating slashes to
 * underscores.
 *   The values of the directory IDs must be in the range 1...63:
 * zero is not allowed because 0 is not valid as a message set number;
 * values larger than 63 cannot be used because only 6 bits of the ID are
 * used in generating kernel trace hookids.
 * Numbers larger than 63 can be used for directories that have message
 * catalogs, but no traces.
 *
 * ==========================================================================
 *
 * DO NOT renumber these IDs, or reuse an ID for a directory that
 * has been deleted, as numbers derived from these IDs appear in the
 * message catalog and trace format file.
 *
 * The message files are listed here with their message number ranges.
 * The range of numbers of MMFS messages 3-digit codes
 * reserved for each existing message catalog file (.msg file)
 *
 * The messages displayed have the format "GPFS: 6027-xxx <message text>"
 * where xxx is the unique 3-digit or 4-digit code for the particular error.
 *
 *
 * SUBDIRECTORY            CATALOG FILE NAME                RANGE OF CODES
 * ------------            -----------------                --------------
 *                                                    unused    000-299
 *  mmfsd                     mmfsd.msg                         300-309
 *  tasking (1)                task.msg                         310-339
 *  classes/tscomm           tscomm.msg                         340-359
 *  disk (1)                   disk.msg                         360-379
 *  classes/basic (1)          base.msg                         380-399
 *  stripe                   stripe.msg                         400-499
 *  vfs                         vfs.msg                         500-529
 *  fs (1)                       fs.msg                         530-599
 *  cfgmgr (1)                 cfgm.msg                         600-649
 *  logger                   logger.msg                         650-659
 *  cfgmgr (2)                 cfgm.msg                         660-799
 *                                                    unused    800-849
 *  util                       util.msg                         850-999
 *  admin (1)                 admin.msg                        1000-1399
 *  log                         log.msg                        1400-1449
 *  defrag                   defrag.msg                        1450-1499
 *  disk (2)                   disk.msg                        1500-1506
 *  admin (2)                 admin.msg                        1507-1529
 *  tasking (2)                task.msg                        1530-1550
 *  admin (3)                 admin.msg                        1551-1699
 *  classes/tscomm (2)       tscomm.msg                        1700-1799
 *  nsd                         nsd.msg                        1800-1849
 *  vdisk (1)                 vdisk.msg                        1850-1899
 *  admin (4)                 admin.msg                        1900-2049
 *  bufmgr                   bufmgr.msg                        2049-2099
 *  admin (4)                 admin.msg                        2100-2499
 *  sanergy                 sanergy.msg                        2500-2549
 *  bufmgr/linux            bufmgr_linux.msg                   2550-2575
 *  tasking/linux           tasking_linux.msg                  2576-2584
 *  tasking/solaris       tasking_solaris.msg                  2585-2599
 *  fs (2)                       fs.msg                        2600-2699
 *  cfgmgr (3)                 cfgm.msg                        2700-2799
 *  pc                           pc.msg                        2800-2899
 *  dm                           dm.msg                        2900-2949
 *  debug                     debug.msg                        (none)
 *  pfsck                     pfsck.msg                        (none)
 *  rs_t3_d11                    (none)                        (none)
 *  rs_t2_d10                    (none)                        (none)
 *  tasking (3)                task.msg                        2950-2999
 *  vdisk (2)                 vdisk.msg                        3000-3099
 *  classes/basic (2)          base.msg                        3100-3119
 *  pcache                   pcache.msg                        3200-3299 
 *  fs (3)                       fs.msg                        3300-3399
 *  flea                       flea.msg                        (none)
 *
 * ==========================================================================
 *
 * $Id: DirIds.h,v 1.112.2.2 2011/08/17 18:04:46 mcnabb Exp $
 *
 * $Log: DirIds.h,v $
 * Revision 1.112.2.2  2011/08/17 18:04:46  mcnabb
 * Defect:807874
 * Do not allow caching of home snapshot files in single-writer fileset, also
 * removed ignoreHomeDirP related changes.
 *
 * Revision 1.112.2.1  2011/06/20 21:34:26  mcnabb
 * Increase the number of traces available to cfgmgr
 *
 * Revision 1.112  2011/04/25 08:25:15  faiz
 *
 * Adding numbers to messages in .msg file.
 *
 * Revision 1.111  2011/04/15 01:12:27  wdavis
 * Change the way tracing numbers are handled to increase the available
 * number of traces for DIR_ID_FS. Make GPFS tracing code aware of the
 * hookid as well as the trace id and change DIR_ID_FS to use SHARKHOOKID=0x309
 *
 * Revision 1.110  2011/02/09 00:38:22  wyllie
 * Move dir ID for vdisk directory to allow 2048 traces.  Invalidates old
 * message catalogs using set number 45.
 *
 * Revision 1.109  2011/01/25 23:37:06  wyllie
 * Switch to new Perseus Reed-Solomon code and new checksum algorithm.
 * To build, VDISK_NEW_CKSUM must be #defined in the makefile, and
 * -DVDISK_NEW_CKSUM must be included in all GPFS compiles.
 *
 * Remove old Perseus RS code, test code, and the rs_TX_dYY directories.
 *
 * Revision 1.108  2010/12/07 23:11:56  tee
 * Use id2name function to trace RDMA operation type.  This will make it
 * easier to add new types.
 *
 * Revision 1.107  2010/08/21 05:37:16  esj
 * Comment change - fix the allocation of message numbers
 *
 * Revision 1.106  2009/12/10 07:18:44  manoj
 * Move pcache files to a new dir ts/pcache. This is initiated by the need to
 * free up trace records in ts/fs, but also consolidates pcache-related code.
 *
 * Revision 1.105  2009/11/10 00:13:52  bryanh
 * Make directory for SCSI stuff, in preparation for proliferation of code for SES
 *
 * Revision 1.104  2009/09/16 15:35:52  bdherr
 * vdisk: Use the mmgetpdisktopolgy script to get the location code and fru
 *   number when creating(or adding to) recovery groups.
 *
 * Revision 1.103  2009/04/30 19:56:06  bdherr
 * vdisk: Reserve more msg numbers for vdisk.msg,  and fix the ones in error.
 *
 * Revision 1.102  2009/02/19 21:10:30  esj
 * Adjust the assigned ranges - tssk.msg was stealing numbers from admin.msg.
 *
 * Revision 1.101  2009/02/19 20:56:05  esj
 * Reserve a new range for tsking/task.msg
 *
 * Revision 1.100  2009/02/17 01:29:21  bcassidy
 * Add directory vdisk/rs_t2_d10.
 *
 * Revision 1.99  2008/12/30 20:39:39  schmuck
 * Have mktrace automatically generate id2name mappings from enum
 * declarations and #define's tagged with special ID2NAME_XXX macros.
 * This eliminates the need to enter and maintain explicit mappings in
 * trcfmt.common.  For details see
 * https://gpfs.almaden.ibm.com/wiki/Trace_facility#Id-to-name_conversions_for_traces_and_dumps
 *
 * Revision 1.98  2007/11/27 23:28:27  manoj
 * Preliminary support for enabling GPFS to function as a caching filesystem.
 * This checkin makes changes to the generic code (not #ifdef'ed by
 * GPFS_CACHE).
 *
 * 1. Support for creating "orphan" directory entries when a READDIR
 * operation encounters a cache miss. For each directory entry that can be
 * specified by a name, assign an inode number and generate the corresponding
 * entry without initializing the actual inode. This will result in the
 * appearance of orphan directory entries in the parent, i.e. entries
 * containing a valid inode number ('in use' according to the allocation map)
 * but lacking the actual inode. Such entries will require special processing
 * during subsequent lookup operations and we will keep track of these entries
 * using a special 'orphan' flag.
 * 2. Fix overlapping trace hook id problem for ts/fs and its subdirs by
 * reserving more id space for ts/fs overflow and moving the id for the
 * subdirs.
 * 3. Pass nameidata (void) pointer to gpfsLookup() in Linux (NULL for other
 * platforms). This will be used subsequently to locate the parent dentry
 * (and its attributes) for performing remote lookup on a cache miss.
 * 4. Pass file (void) pointer to gpfsOpen() in Linux (NULL for other platforms).
 * This will be saved subsequently for perform remote IO operations (using
 * the dentry from the file structure to obtain attributes).
 *
 * Revision 1.96  2007/05/03 17:31:46  yuri
 * Update comment reflecting the use of id space in ts/fs.
 *
 * Revision 1.95  2007/04/25 00:18:30  mcnabb
 * Document the required DIR_ID immutability necessary for backward
 * compatibility of message catalog usage.
 * Add usage of set 60 by admin messages.
 *
 * Revision 1.94  2006/12/14 19:00:10  bhartner
 * verbs RDMA support for Linux
 *
 * Revision 1.93  2006/10/03 14:20:50  bhartner
 * Add RDMA event threads, mutex, condvar, and trace.
 *
 * Revision 1.92  2006/09/19 16:20:08  sberman
 * Numerous small changes for compilation on Solaris.  Temporary fixes in
 * SANERGY for compilation, though may get deleted later.  DMAPI FID porting.
 *
 * Revision 1.91  2006/08/22 23:59:42  wyllie
 * Add vdisk subdirectory
 *
 * Revision 1.90  2006/08/11 00:47:20  mcnabb
 * Tweak this so that renumbered DIR_ID_s are in order of new assignment.
 *
 * Revision 1.89  2006/08/10 22:32:10  yuri
 * Fix overlapping trace hook id problem for ts/kernext and its subdirs by
 * reserving some id space for ts/kernext overflow.
 *
 * Revision 1.88  2006/07/26 23:36:27  mcnabb
 * Clean up DIR_ID_* assignments so that platform specific subdirs use
 * the same number as the other platforms.
 * Document the *.msg files that implicitly use DIR_ID_* assignments.
 *
 * Revision 1.87  2006/07/17 17:47:06  sberman
 * Initial check-in for Solaris GPFS porting changes.
 *
 * Revision 1.86  2006/04/28 15:01:13  gjertsen
 * Merge win32a branch into main cvs branch sans admin script changes;
 * admin script mods will follow in the near future.
 *
 * Revision 1.85  2006/04/03 19:25:58  makaplan
 * Claim message numbers 2800-2899 for policy, ts/pc/pc.msg
 *
 * Revision 1.84  2006/03/10 06:29:58  schmuck
 * Prune check-in history.  No code change.
 *
 * Revision 1.83  2006/03/02 03:25:16  hempel
 * message numbering:  add new message ranges for the fs and cfgmgr directories
 *
 * Revision 1.82.2.1  2006/02/10 22:39:31  demyn
 * added ids for XXXX_WINDOWS directories
 *
 * Revision 1.82  2005/11/15 16:26:42  gjertsen
 * Provide a helpful message when daemon encounters an out of memory
 * situation along with more graceful shutdown (defect 518290).
 *
 * Revision 1.81  2005/08/12 19:39:41  gjertsen
 * Fix for 515602. Provide useful error message and shutdown if allocation
 * fails. Create msg catalog file for bufmgr/linux but reuse MSG_E_NOMEM.
 *
 * Revision 1.80  2005/05/03 16:48:44  makaplan
 * POLICY - integration tweaks
 *
 * Revision 1.79  2005/01/15 00:58:41  mcnabb
 * Remove reference to smit messages
 *
 * [ 74 log messages deleted ]
 *
 * Revision 1.1  1993/07/16  18:09:11  wyllie
 * Add file with unique IDs for each Shark source code directory.
 *
 */

/*
 * IMPORTANT (see also comments at the top of this file):
 *  - names must be of the form DIR_ID_XXX, where XXX = relative dir name
 *  - values must be in the range 1 ... 63
 * Subdirs for different architectures can reuse the same number since they
 * are never combined in the same build.
 * If a *.msg file is mentioned, then there is a "$set nn" line in that file
 * and the numbers MUST match.
 *
 * Do not renumber any existing DIR_ID if there are *.msg files that use
 * that number. These must be backward compatible for releases 3.1 or later.
 * This applies to any messages the GPFS daemon may look up from a client node.
 * Utilities that only do message lookup locally do not have this restriction.
 *
 */
#define DIR_ID_CFGMGR                   1    /* cfgmgr/trcid.h cfgm.msg */
#define DIR_ID_CLASSES_BASIC            2    /* classes/basic/trcid.h base.msg */
/* unused                               3 */
#define DIR_ID_LOCKMGR                  4    /* lockmgr/trcid.h */
#define DIR_ID_CLASSES_TSCOMM           5    /* classes/tscomm/trcid.h tscomm.msg */
#define DIR_ID_DEBUG                    6    /* debug.msg */
#define DIR_ID_DISK                     7    /* disk/trcid.h disk.msg */

#define DIR_ID_FS                       8    /* fs/trcid.h fs.msg */
/* DIR_ID_FS overflow. do not reuse.    9 */ /* (fs/trcid.h) */
/* DIR_ID_FS overflow. do not reuse.    10 *//* (fs/trcid.h) */
/* DIR_ID_FS overflow. do not reuse.    11 *//* (fs/trcid.h) */

#define DIR_ID_LOGGER                   12   /* logger/trcid.h logger.msg */
#define DIR_ID_MMFSD                    13   /* mmfsd/trcid.h mmfsd.msg */
#define DIR_ID_ADMIN                    14   /* admin.msg */
#define DIR_ID_PAGEMGR                  15   /* pagemgr/trcid.h */
#define DIR_ID_KERNEXT_GPL_LINUX        16   /* kernext/gpl-linux/trcid.h */

#define DIR_ID_KERNEXT_GPL_LINUX_I386   17
#define DIR_ID_KERNEXT_GPL_LINUX_IA64   17   /* kernext/gpl-linux/ia64/trcid.h */

#define DIR_ID_TASKING                  18   /* tasking/trcid.h task.msg */
#define DIR_ID_VFS                      19   /* vfs.msg */
#define DIR_ID_KERNEXT                  20   /* kernext/trcid.h */
/* DIR_ID_KERNEXT overflow. do not use. 21 */ /* kernext/trcid.h */
#define DIR_ID_DM                       22   /* dm/trcid.h dm.msg */
#define DIR_ID_DM_KX                    23   /* dm/kx/trcid.h */

#define DIR_ID_DM_KX_AIX                24   /* dm/kx/aix/trcid.h */
#define DIR_ID_DM_KX_LINUX              24   /* dm/kx/linux/trcid.h */
#define DIR_ID_DM_KX_WINDOWS            24   /* dm/kx/windows/trcid.h */
#define DIR_ID_DM_KX_SOLARIS            24   /* dm/kx/solaris/trcid.h */

#define DIR_ID_LOG                      25   /* log/trcid.h log.msg */
#define DIR_ID_CLASSES_PERFMON          26   /* classes/perfmon/trcid.h */
#define DIR_ID_UTIL                     27   /* util.msg */
#define DIR_ID_STRIPE                   28   /* stripe/trcid.h stripe.msg */

#define DIR_ID_KERNEXT_AIX              29   /* kernext/aix/trcid.h */
#define DIR_ID_KERNEXT_LINUX            29   /* kernext/linux/trcid.h */
#define DIR_ID_KERNEXT_WINDOWS          29   /* kernext/windows/trcid.h */
#define DIR_ID_KERNEXT_SOLARIS          29

#define DIR_ID_FS_AIX                   30   /* fs/aix/trcid.h */
#define DIR_ID_FS_LINUX                 30   /* fs/linux/trcid.h */
#define DIR_ID_FS_WINDOWS               30   /* fs/windows/trcid.h */
#define DIR_ID_FS_SOLARIS               30

/* unused                               31 */
#define DIR_ID_ADMIN_SCRIPT             32   /* admin.msg */
#define DIR_ID_TM                       33   /* tm/trcid.h */
/* unused                               34 */
#define DIR_ID_DEFRAG                   36   /* defrag/trcid.h defrag.msg */
#define DIR_ID_CLASSES_MBPERFMON        37   /* classes/mbperfmon/trcid.h */
#define DIR_ID_PFSCK                    38   /* pfsck/trcid.h pfsck.msg */
#define DIR_ID_VDISK                    39   /* vdisk/trcid.h vdisk.msg */
/* DIR_ID_VDISK overflow. do not reuse. 40 *//* (vdisk/trcid.h) */
/* unused - try to reserve for DIR_ID_VDISK overflow if possible.  41 */
/* unused                               42 */
/* unused                               43 */
/* unused                               44 */
/* unused                               45 *//* former vdisk.msg; can reuse */
#define DIR_ID_NSD                      46   /* nsd/trcid.h nsd.msg */

#define DIR_ID_LOGGER_AIX               47   /* logger/aix/trcid.h */
#define DIR_ID_LOGGER_LINUX             47   /* logger/linux/trcid.h */
#define DIR_ID_LOGGER_WINDOWS           47   /* logger/windows/trcid.h */
#define DIR_ID_LOGGER_SOLARIS           47

#define DIR_ID_PCACHE                   48   /* pcache/trcid.h */
/* unused                               49 */
/* unused                               50 */

#define DIR_ID_TASKING_AIX              51   /* tasking/aix/trcid.h */
#define DIR_ID_TASKING_LINUX            51   /* tasking/linux/trcid.h tasking_linux.msg */
#define DIR_ID_TASKING_WINDOWS          51   /* tasking/windows/trcid.h */
#define DIR_ID_TASKING_SOLARIS          51   /* tasking_solaris.msg */

#define DIR_ID_PC                       52   /* pc/trcid.h pc.msg */
#define DIR_ID_SANERGY                  53   /* sanergy/trcid.h sanergy.msg */
#define DIR_ID_SANERGY_KX               54   /* sanergy/kx/trcid.h */
#define DIR_ID_BUFMGR                   55   /* bufmgr/trcid.h bufmgr.msg */

#define DIR_ID_BUFMGR_AIX               56   /* bufmgr/aix/trcid.h */
#define DIR_ID_BUFMGR_LINUX             56   /* bufmgr/linux/trcid.h bufmgr_linux.msg */
#define DIR_ID_BUFMGR_WINDOWS           56   /* bufmgr/windows/trcid.h */
#define DIR_ID_BUFMGR_SOLARIS           56   /* bufmgr/solaris/trcid.h */

#define DIR_ID_RDMA_UDAPL               57   /* rdma/udapl/trcid.h */
#define DIR_ID_RDMA_VERBS               58   /* rdma/verbs/trcid.h */
#define DIR_ID_RDMA                     59   /* rdma/trcid.h */

#define DIR_ID_ADMIN_MORE               60   /* admin.msg */
/* unused                               61 */
/* unused                               62 */
/* unused                               63 */
/* END OF VALID ID RANGE */

/* Add new directory IDs above here */

/* Dummy DIR_IDs for directories that do not contain any message catalogs or
   traces, but have id2name definitions that must be extracted my mktrace */
#define DIR_ID_KERNEXT_IBM_KXI           0

/* overrides for default hookid (307) in certain directories */
/* To set a new one, you just add a SHARKHOOKID_DIRNAME */
#define SHARKHOOKID_TM      0x30600000
#define SHARKHOOKID_FS      0x30900000
#define SHARKHOOKID_CFGMGR  0x30800000
#define SHARKHOOKID_TASKING 0x30800000
#define SHARKHOOKID_KERNEXT 0x30900000
#define SHARKHOOKID \
  (LOCAL_BASE == DIR_ID_TM ? SHARKHOOKID_TM : \
   LOCAL_BASE == DIR_ID_TASKING ? SHARKHOOKID_TASKING : \
   LOCAL_BASE == DIR_ID_FS ? SHARKHOOKID_FS : \
   LOCAL_BASE == DIR_ID_KERNEXT ? SHARKHOOKID_KERNEXT : \
   LOCAL_BASE == DIR_ID_CFGMGR ? SHARKHOOKID_CFGMGR : \
   0x30700000)


#endif  /* _h_DirIds */
